
public class ArrayBasedList<E extends Comparable<? super E>> implements ListInterface<E>{

	private final int DEFAULT_ARRAY_SIZE = 100;

	private int count;
	private E[] array;

	public ArrayBasedList() {
		this.count = 0;
		this.array = (E[]) new Comparable[DEFAULT_ARRAY_SIZE];
	}

	public ArrayBasedList(int size) {
		this.count = 0;
		if (size > 0) {
			this.array = (E[]) new Comparable[size];
		} else {
			this.array = (E[]) new Comparable[DEFAULT_ARRAY_SIZE];
		}
	}
	
	@Override
	public int size() {
		return this.count;
	}
	
	@Override
	public boolean isEmpty() {
		return (this.count == 0);
	}
	
	@Override
	public void add(E element) {
	 	if (this.count == this.array.length) {
			increaseArraySize();
		}
		this.array[this.count++] = element;
	}
	
	@Override
	public void add(E element, int index) throws IndexOutOfBoundsException {
		if ((index < 0) || (index > this.count)) {
			throw new IndexOutOfBoundsException("index = " + index + " is invalid");
		} else if (this.count == this.array.length) {
			increaseArraySize();
		} 
		
		if (index == this.count) {
			this.add(element);
		} else {
			for ( int i = this.count ; i > index ; i-- ) {
				this.array[i] = this.array[i-1];
			}
			this.array[index] = element;
			this.count++;
		}
	}

	@Override
	public E get(int index) throws IndexOutOfBoundsException {
		if ((index >= 0) && (index < this.count)) {
			return this.array[index];
		} else {
			throw new IndexOutOfBoundsException("index = " + index + " is invalid");
		}
	}

	@Override
	public E replace(E element, int index) throws IndexOutOfBoundsException {
		if ((index >= 0) && (index < this.count)) {
			E oldElement = this.array[index];
			this.array[index] = element;
			return oldElement;
		} else {
			throw new IndexOutOfBoundsException("index = " + index + " is invalid");
		}		
	}
	
	@Override
	public E remove(int index) throws IndexOutOfBoundsException {
		if ((index < 0) || (index > (this.count - 1))) {
			throw new IndexOutOfBoundsException("index = " + index + " is invalid");
		} else {
			E element = this.array[index];
			for ( int i = index ; i < (this.count - 1) ; i++ ) {
				this.array[i] = this.array[i+1];
			}
			this.count--;
			return element;
		}
	}
	
	@Override
	public void removeAll() {
		for ( int i = 0 ; i < this.array.length ; i++ ) {
			this.array[i] = null;
		}
		this.count = 0;
	}
	
	
	private void increaseArraySize() {
		E[] newArray = (E[]) new Comparable[this.array.length + this.DEFAULT_ARRAY_SIZE];
		for ( int i = 0 ; i < this.array.length ; i++ ) {
			newArray[i] = this.array[i];
		}
		this.array = newArray;
	}

	@Override
	public String toString() {
		String s = "array = {";
		for ( int i = 0 ; i < this.count ; i++ ) {
			if ( i < this.count - 1) {
				s = s + this.array[i] + ", ";
			} else {
				s = s + this.array[i] + "}";
			}
		}
		return s;
	}

	@Override
	public ListInterface<E> copy() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void addSorted(E element) {
		// TODO Auto-generated method stub
		
	}

	
	
}
